#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-05-20

# based on https://github.com/allangood/check_mk/tree/master/plugins/squid
#
# rewritten for cmk 2.x
#
# 2023-05-20: moved to ~/local/lib/check_mk/base/plugins/agent_based/utils
#             renamed from check_squid.include to squid.py
#


# !/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-05-20

# based on https://github.com/allangood/check_mk/tree/master/plugins/squid by Allan GooD: allan.cassaro@gmail.com
#
# rewritten for cmk 2.x
#

# ToDo: add support for multiple instances

from typing import Dict
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    check_levels,
    check_levels_predictive,
    Service,
    render
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
)


def parse_squid(string_table) -> Dict[str, float]:
    section = {}
    for line in string_table:
        entry, value = line[0].split('=')
        entry = entry.strip()
        try:
            value = float(value.split('/')[0].strip().split(' ')[0].split('%')[0])
        except ValueError:
            continue
        if 'kbytes' in entry:
            value = value * 1024 + 8 # change kbytes to bits
        section[entry] = value

    return section


register.agent_section(
    name="squid",
    parse_function=parse_squid,
)


def render_per_second(value: float) -> str:
    return f'{value:0.2f}/s'


squid_map = {
    'Client': {
        'client_http.requests': {
            'label': 'Requests',
            'levels_upper': 'client_requests',
            'metric_name': 'client_requests_per_second',
            'render_func': render_per_second,
        },
        'client_http.errors': {
            'label': 'Errors',
            'levels_upper': 'client_errors',
            'metric_name': 'client_errors_per_second',
            'render_func': render_per_second,
        },
        'client_http.hits': {
            'label': 'Cache Hits',
            'levels_upper': 'client_hits',
            'metric_name': 'client_hits_per_second',
            'render_func': render_per_second,
        },
        'client_http.kbytes_in': {
            'label': 'In',
            'levels_upper': 'client_bytes_in_out',
            'metric_name': 'if_in_bps',
            'render_func': render.networkbandwidth,
        },
        'client_http.kbytes_out': {
            'label': 'Out',
            'levels_upper': 'client_bytes_in_out',
            'metric_name': 'if_out_bps',
            'render_func': render.networkbandwidth,
        },
    },
    'Server': {
        'server.all.requests': {
            'label': 'Requests',
            'levels_upper': 'server_requests',
            'metric_name': 'server_all_requests',
            'render_func': render_per_second,
        },
        'server.all.errors': {
            'label': 'Errors',
            'levels_upper': 'server_errors',
            'metric_name': 'server_all_errors',
            'render_func': render_per_second,
        },
        'server.all.kbytes_in': {
            'label': 'In',
            'levels_upper': 'server_bytes_in_out',
            'metric_name': 'if_in_bps',
            'render_func': render.networkbandwidth,
        },
        'server.all.kbytes_out': {
            'label': 'Out',
            'levels_upper': 'server_bytes_in_out',
            'metric_name': 'if_out_bps',
            'render_func': render.networkbandwidth,
        },
    },
    'CPU': {
        'cpu_usage': {
            'label': 'Usage',
            'levels_upper': 'cpu_usage',
            'metric_name': 'util',
            'render_func': render.percent,
        },
        'cpu_time': {
            'label': 'Time',
            'levels_upper': 'cpu_time',
            'metric_name': 'cpu_time',
            'render_func': render.timespan,
        },
    },
    'DNS': {
        'dns.median_svc_time': {
            'label': 'Median response time',
            'levels_upper': 'dns_time',
            'metric_name': 'dns_median_svc_time',
            'render_func': render.timespan,
        }},
}


def discovery_squid(section: Dict[str, float]) -> DiscoveryResult:
    for item in squid_map.keys():
        yield Service(item=item)


def check_squid(item, params, section: Dict[str, float]) -> CheckResult:
    for value in squid_map[item].keys():
        entry = squid_map[item][value]
        levels_upper = params.get(entry['levels_upper'], None)
        yield from  check_levels_predictive(
            label=entry['label'],
            value=section[value],
            levels=levels_upper,
            render_func=entry['render_func'],
            metric_name=entry['metric_name'],
        ) if isinstance(levels_upper, dict) else check_levels(
            label=entry['label'],
            value=section[value],
            levels_upper=levels_upper,
            render_func=entry['render_func'],
            metric_name=entry['metric_name'],
        )


register.check_plugin(
    name='squid',
    service_name='Squid %s',
    discovery_function=discovery_squid,
    check_function=check_squid,
    check_default_parameters={
        # 'client_requests': (600, 800),
        # 'client_hits': (600, 800),
        # 'server_requests': (600, 800),
        'dns_time': (2, 4),
        'cpu_usage': (60, 80)
    },
    check_ruleset_name='squid',
)
